@@ -23,6 +23,8 @@
 #include "ctree.h"
 #include <dirent.h>
 #include <stdarg.h>
+#include "internal.h"
+#include "btrfs-list.h"
 
 #define BTRFS_MKFS_SYSTEM_GROUP_SIZE (4 * 1024 * 1024)
 #define BTRFS_MKFS_SMALL_VOLUME_SIZE (1024 * 1024 * 1024)
@@ -109,19 +111,23 @@ void btrfs_parse_features_to_string(char *buf, u64 flags);
 struct btrfs_mkfs_config {
 	char *label;
 	char *fs_uuid;
+	char *chunk_uuid;
 	u64 blocks[8];
 	u64 num_bytes;
 	u32 nodesize;
 	u32 sectorsize;
 	u32 stripesize;
 	u64 features;
+
+	/* Super bytenr after make_btrfs */
+	u64 super_bytenr;
 };
 
 int make_btrfs(int fd, struct btrfs_mkfs_config *cfg);
 int btrfs_make_root_dir(struct btrfs_trans_handle *trans,
 			struct btrfs_root *root, u64 objectid);
-int btrfs_prepare_device(int fd, char *file, int zero_end, u64 *block_count_ret,
-			 u64 max_block_count, int discard);
+int btrfs_prepare_device(int fd, const char *file, int zero_end,
+		u64 *block_count_ret, u64 max_block_count, int discard);
 int btrfs_add_to_fsid(struct btrfs_trans_handle *trans,
 		      struct btrfs_root *root, int fd, char *path,
 		      u64 block_count, u32 io_width, u32 io_align,
@@ -154,7 +160,7 @@ int get_fs_info(char *path, struct btrfs_ioctl_fs_info_args *fi_args,
 int get_label(const char *btrfs_dev, char *label);
 int set_label(const char *btrfs_dev, const char *label);
 
-char *__strncpy__null(char *dest, const char *src, size_t n);
+char *__strncpy_null(char *dest, const char *src, size_t n);
 int is_block_device(const char *file);
 int is_mount_point(const char *file);
 int check_arg_type(const char *input);
@@ -162,17 +168,17 @@ int open_path_or_dev_mnt(const char *path, DIR **dirstream, int verbose);
 int btrfs_open_dir(const char *path, DIR **dirstream, int verbose);
 u64 btrfs_device_size(int fd, struct stat *st);
 /* Helper to always get proper size of the destination string */
-#define strncpy_null(dest, src) __strncpy__null(dest, src, sizeof(dest))
-int test_dev_for_mkfs(char *file, int force_overwrite);
+#define strncpy_null(dest, src) __strncpy_null(dest, src, sizeof(dest))
+int test_dev_for_mkfs(const char *file, int force_overwrite);
 int get_label_mounted(const char *mount_path, char *labelp);
 int get_label_unmounted(const char *dev, char *label);
 int test_num_disk_vs_raid(u64 metadata_profile, u64 data_profile,
-	u64 dev_cnt, int mixed);
+	u64 dev_cnt, int mixed, int ssd);
 int group_profile_max_safe_loss(u64 flags);
-int is_vol_small(char *file);
+int is_vol_small(const char *file);
 int csum_tree_block(struct btrfs_root *root, struct extent_buffer *buf,
 			   int verify);
-int ask_user(char *question);
+int ask_user(const char *question);
 int lookup_ino_rootid(int fd, u64 *rootid);
 int btrfs_scan_lblkid(void);
 int get_btrfs_mount(const char *dev, char *mp, size_t mp_size);
@@ -180,17 +186,19 @@ int find_mount_root(const char *path, char **mount_root);
 int get_device_info(int fd, u64 devid,
 		struct btrfs_ioctl_dev_info_args *di_args);
 int test_uuid_unique(char *fs_uuid);
-u64 disk_size(char *path);
+u64 disk_size(const char *path);
 int get_device_info(int fd, u64 devid,
 		struct btrfs_ioctl_dev_info_args *di_args);
-u64 get_partition_size(char *dev);
-const char* group_type_str(u64 flags);
-const char* group_profile_str(u64 flags);
+u64 get_partition_size(const char *dev);
 
 int test_minimum_size(const char *file, u32 leafsize);
 int test_issubvolname(const char *name);
+int test_issubvolume(const char *path);
 int test_isdir(const char *path);
 
+const char *subvol_strip_mountpoint(const char *mnt, const char *full_path);
+int get_subvol_info(const char *fullpath, struct root_info *get_ri);
+
 /*
  * Btrfs minimum size calculation is complicated, it should include at least:
  * 1. system group size
@@ -212,8 +220,8 @@ static inline u64 btrfs_min_dev_size(u32 leafsize)
 }
 
 int find_next_key(struct btrfs_path *path, struct btrfs_key *key);
-char* btrfs_group_type_str(u64 flag);
-char* btrfs_group_profile_str(u64 flag);
+const char* btrfs_group_type_str(u64 flag);
+const char* btrfs_group_profile_str(u64 flag);
 
 /*
  * Get the length of the string converted from a u64 number.
@@ -247,7 +255,7 @@ int btrfs_check_nodesize(u32 nodesize, u32 sectorsize, u64 features);
 
 const char *get_argv0_buf(void);
 
-#define HELPINFO_OUTPUT_UNIT							\
+#define HELPINFO_UNITS_LONG							\
 	"--raw              raw numbers in bytes",				\
 	"--human-readable   human friendly numbers, base 1024 (default)",	\
 	"--iec              use 1024 as a base (KiB, MiB, GiB, TiB)",		\
@@ -257,7 +265,7 @@ const char *get_argv0_buf(void);
 	"--gbytes           show sizes in GiB, or GB with --si",		\
 	"--tbytes           show sizes in TiB, or TB with --si"
 
-#define HELPINFO_OUTPUT_UNIT_DF							\
+#define HELPINFO_UNITS_SHORT_LONG						\
 	"-b|--raw           raw numbers in bytes",				\
 	"-h|--human-readable",							\
 	"                   human friendly numbers, base 1024 (default)",	\
@@ -270,8 +278,10 @@ const char *get_argv0_buf(void);
 	"-t|--tbytes        show sizes in TiB, or TB with --si"
 
 unsigned int get_unit_mode_from_arg(int *argc, char *argv[], int df_mode);
+void clean_args_no_options(int argc, char *argv[], const char * const *usage);
 int string_is_numerical(const char *str);
 
+__attribute__ ((format (printf, 1, 2)))
 static inline void warning(const char *fmt, ...)
 {
 	va_list args;
@@ -283,6 +293,7 @@ static inline void warning(const char *fmt, ...)
 	fputc('\n', stderr);
 }
 
+__attribute__ ((format (printf, 1, 2)))
 static inline void error(const char *fmt, ...)
 {
 	va_list args;
@@ -294,6 +305,7 @@ static inline void error(const char *fmt, ...)
 	fputc('\n', stderr);
 }
 
+__attribute__ ((format (printf, 2, 3)))
 static inline int warning_on(int condition, const char *fmt, ...)
 {
 	va_list args;
@@ -310,6 +322,7 @@ static inline int warning_on(int condition, const char *fmt, ...)
 	return 1;
 }
 
+__attribute__ ((format (printf, 2, 3)))
 static inline int error_on(int condition, const char *fmt, ...)
 {
 	va_list args;
